==> ask-cli-1.5.0.tgz: package/lib/utils/lambda-operation/lambda-endpoint-utility.js <==
const fs = require('fs');
const path = require('path');
const CONSTANTS = require('../constants');
const jsonUtility = require('../json-utility');

module.exports = {
    parseLambdaArn,
    getLambdaFolderName,
    removeLATESTFromArn,
    generateLambdaResourceList,
    resolveLambdaAwsRegion,
    isLambdaUsingAskSdk
};

/**
 * parse the given URI.
 *
 * if it's a vilad lambda ARN, it returns an object:
 * {
 *      'arn': uri without $LATEST if applicable,
 *      'awsRegion': aws region,
 *      'functionName': lambda function name,
 *      'version': the lambda version or 'null'
 * }
 * @param uri
 * @return {{arn: *, awsRegion: *, functionName: *, version: (*|null)}} | null
 */
function parseLambdaArn(uri) {
    // regex result should be:
    // [ arn, 'arn:aws:lambda:', region, 12Digits, functionName, ':version', version]
    const regexResult = CONSTANTS.REGEX.LAMBDA.exec(uri);
    if (!regexResult) { return; }

    // edge case. arn:$LATEST. remove the $LATEST
    if (regexResult[6] && regexResult[6] === '$LATEST') {
        uri = uri.slice(0, -8);
    }

    return {
        'arn': uri,
        'awsRegion': regexResult[2],
        'functionName': regexResult[4],
        'version': regexResult[6] || null
    };
}

function getLambdaFolderName(uri) {
    const parseResult = parseLambdaArn(uri);
    return `${parseResult.awsRegion}_${parseResult.functionName}`;
}

/**
 * remove '$LATEST' from lambda arn if applicable
 *
 * @param uri
 * @return {*}
 */
function removeLATESTFromArn(uri) {
    if (uri && uri.slice(-8) === ':$LATEST') {
        return uri.slice(0, -8);
    }
    return uri;
}

/**
 * Generate lambda resources from endpoint
 * the format is:
 * {
 *  'arn': lambdaArn,
 *  'awsRegion': 'us-east-1',
 *  'functionName': functionName
 *  'alexaUsage': ["domain/region", "domain/region", ...],
 *  'revisionId': ""
 * }
 * @param endPointsInfo
 * @return {Array}
 * @private
 */
function generateLambdaResourceList(endPointsInfo) {
    const lambdaMap = {};
    for (const domain of Object.keys(endPointsInfo)) {
        for (const region of Object.keys(endPointsInfo[domain])) {
            const uri = endPointsInfo[domain][region].uri;
            const lambdaParseInfo = parseLambdaArn(uri);
            if (lambdaParseInfo && !lambdaMap[lambdaParseInfo.arn]) {
                lambdaMap[lambdaParseInfo.arn] = {
                    'arn': lambdaParseInfo.arn,
                    'awsRegion': lambdaParseInfo.awsRegion,
                    'functionName': lambdaParseInfo.functionName,
                    'alexaUsage': [],
                    'revisionId': ''
                };
            }
            lambdaMap[lambdaParseInfo.arn].alexaUsage.push(`${domain}/${region}`);
        }
    }

    // transform from map to list
    return Object.keys(lambdaMap).map(arn => lambdaMap[arn]);
}

function resolveLambdaAwsRegion(region) {
    return CONSTANTS.REGION_MAPPING[region] || CONSTANTS.REGION_MAPPING.NA;
}

function isLambdaUsingAskSdk(sourceDir) {
    const packagePath = path.join(sourceDir, 'package.json');
    if (fs.existsSync(packagePath)) {
        const packageInfo = jsonUtility.read(packagePath);
        if (jsonUtility.getPropertyValueFromObject(packageInfo, ['dependencies', 'ask-sdk-core'])
            && jsonUtility.getPropertyValueFromObject(packageInfo, ['dependencies', 'ask-sdk-model'])) {
            return true;
        }
    }
    return false;
}

==> ask-cli-1.6.0.tgz: package/lib/utils/lambda-operation/lambda-endpoint-utility.js <==
const fs = require('fs');
const path = require('path');
const CONSTANTS = require('@src/utils/constants');
const jsonUtility = require('@src/utils/json-utility');
const urlUtils = require('@src/utils/url-utils');
const initAws = require('@src/utils/init-aws');

module.exports = {
    parseLambdaArn,
    getLambdaFolderName,
    removeLATESTFromArn,
    generateLambdaResourceList,
    resolveLambdaAwsRegion,
    isLambdaUsingAskSdk
};

/**
 * parse the given URI.
 *
 * if it's a vilad lambda ARN, it returns an object:
 * {
 *      'arn': uri without $LATEST if applicable,
 *      'awsRegion': aws region,
 *      'functionName': lambda function name,
 *      'version': the lambda version or 'null'
 * }
 * @param uri
 * @return {{arn: *, awsRegion: *, functionName: *, version: (*|null)}} | null
 */
function parseLambdaArn(uri) {
    // regex result should be:
    // [ arn, 'arn:aws:lambda:', region, 12Digits, functionName, ':version', version]
    const regexResult = CONSTANTS.REGEX.LAMBDA.exec(uri);
    if (!regexResult) { return; }

    // edge case. arn:$LATEST. remove the $LATEST
    if (regexResult[6] && regexResult[6] === '$LATEST') {
        uri = uri.slice(0, -8);
    }

    return {
        'arn': uri,
        'awsRegion': regexResult[2],
        'functionName': regexResult[4],
        'version': regexResult[6] || null
    };
}

function getLambdaFolderName(uri) {
    const parseResult = parseLambdaArn(uri);
    return `${parseResult.awsRegion}_${parseResult.functionName}`;
}

/**
 * remove '$LATEST' from lambda arn if applicable
 *
 * @param uri
 * @return {*}
 */
function removeLATESTFromArn(uri) {
    if (uri && uri.slice(-8) === ':$LATEST') {
        return uri.slice(0, -8);
    }
    return uri;
}

/**
 * Generate lambda resources from endpoint
 * the format is:
 * {
 *  'arn': lambdaArn,
 *  'awsRegion': 'us-east-1',
 *  'functionName': functionName
 *  'alexaUsage': ["domain/region", "domain/region", ...],
 *  'revisionId': ""
 * }
 * @param endPointsInfo
 * @return {Array}
 * @private
 */
function generateLambdaResourceList(endPointsInfo) {
    const lambdaMap = {};
    for (const domain of Object.keys(endPointsInfo)) {
        for (const region of Object.keys(endPointsInfo[domain])) {
            const uri = endPointsInfo[domain][region].uri;
            const lambdaParseInfo = parseLambdaArn(uri);
            if (lambdaParseInfo && !lambdaMap[lambdaParseInfo.arn]) {
                lambdaMap[lambdaParseInfo.arn] = {
                    'arn': lambdaParseInfo.arn,
                    'awsRegion': lambdaParseInfo.awsRegion,
                    'functionName': lambdaParseInfo.functionName,
                    'alexaUsage': [],
                    'revisionId': ''
                };
            }
            lambdaMap[lambdaParseInfo.arn].alexaUsage.push(`${domain}/${region}`);
        }
    }

    // transform from map to list
    return Object.keys(lambdaMap).map(arn => lambdaMap[arn]);
}

/**
 * Lambda function region resolving chain.
 * Extract the awsRegion from an existing Lambda ARN or
 * Check region mapping or
 * Check user's AWS profile setting or
 * Use us-east-1 as the value
 * @param {String} arn      Lambda ARN (set null if creating Lambda)
 * @param {String} region   The Alexa region for the Lambda
 * @param {String} awsProfile
 */
function resolveLambdaAwsRegion(arn, region, awsProfile) {
    if (urlUtils.isLambdaArn(arn)) {
        return parseLambdaArn(arn).awsRegion;
    }

    return CONSTANTS.REGION_MAPPING[region] || initAws.getCLICompatibleDefaultRegion(awsProfile) || CONSTANTS.REGION_MAPPING.NA;
}

function isLambdaUsingAskSdk(sourceDir) {
    const packagePath = path.join(sourceDir, 'package.json');
    if (fs.existsSync(packagePath)) {
        const packageInfo = jsonUtility.read(packagePath);
        if (jsonUtility.getPropertyValueFromObject(packageInfo, ['dependencies', 'ask-sdk-core'])
            && jsonUtility.getPropertyValueFromObject(packageInfo, ['dependencies', 'ask-sdk-model'])) {
            return true;
        }
    }
    return false;
}
